@@ -8,6 +8,8 @@ from talos.thrift.topic.ttypes import GetTopicAttributeRequest
 from talos.thrift.topic.ttypes import GetTopicAttributeResponse
 from talos.thrift.message.ttypes import MessageType
 from talos.client.Constants import Constants
+from talos.thrift.topic.ttypes import GetDescribeInfoRequest
+from talos.client.ScheduleInfoCache import ScheduleInfoCache
 from atomic import AtomicLong
 from talos.producer.BufferedMessageCount import BufferedMessageCount
 from talos.producer.UserMessage import UserMessage
@@ -18,12 +20,12 @@ from talos.admin.TalosAdmin import TalosAdmin
 from talos.utils import Utils
 from talos.utils.Utils import synchronized
 from talos.producer.PartitionSender import PartitionSender
-from talos.thrift.common.ttypes import GalaxyTalosException
+from talos.utils.FalconWriter import FalconWriter
 from threading import Timer
 import threading
 import random
-import logging
 import traceback
+import logging
 
 
 class PRODUCER_STATE:
@@ -36,22 +38,26 @@ class TalosProducer(object):
     logger = logging.getLogger("TalosProducer")
     partitionKeyMinLen = Constants.TALOS_PARTITION_KEY_LENGTH_MINIMAL
     partitionKeyMaxLen = Constants.TALOS_PARTITION_KEY_LENGTH_MAXIMAL
-    requestId = AtomicLong(1)
-    globalLock = threading.Condition()
     partitionNumber = int
     curPartitionId = int
 
     messageCallbackExecutors = threading.Thread
     partitionCheckExecutor = threading.Timer
-    partitionSenderMap = dict()
+
+    falconWriter = FalconWriter
+    producerMonitorThread = threading.Timer
 
     lock = threading.Lock()
 
     def __init__(self, producerConfig=None, credential=None, topicName=None, talosAdmin=None,
                  talosClientFactory=None, topicTalosResourceName=None, partitioner=None,
                  topicAbnormalCallback=None, userMessageCallback=None):
+        self.requestId = AtomicLong(1)
+        self.globalLock = threading.Condition()
+        self.partitionSenderMap = dict()
         self.talosProducerConfig = producerConfig
         self.producerState = PRODUCER_STATE.ACTIVE
+        self.topicName = topicName
         if talosClientFactory:
             self.talosClientFactory = talosClientFactory
         else:
@@ -60,12 +66,11 @@ class TalosProducer(object):
             self.talosAdmin = talosAdmin
         else:
             self.talosAdmin = TalosAdmin(self.talosClientFactory)
-        self.topicName = topicName
         if topicTalosResourceName:
             self.topicTalosResourceName = topicTalosResourceName
         else:
-            self.topicTalosResourceName = self.talosAdmin.get_topic_attribute(
-                GetTopicAttributeRequest(topicName=self.topicName))
+            self.topicTalosResourceName = self.talosAdmin.get_describe_info(
+                GetDescribeInfoRequest(self.topicName))
         self.partitioner = partitioner
         self.topicAbnormalCallback = topicAbnormalCallback
         self.userMessageCallback = userMessageCallback
@@ -80,12 +85,21 @@ class TalosProducer(object):
                                                   self.maxBufferedMsgBytes)
         self.clientId = Utils.generate_client_id(producerConfig.get_client_ip(), "producer")
         self.synchronizedLock = threading.Lock()
+        # get scheduleInfo
+        self.scheduleInfoCache = ScheduleInfoCache().get_schedule_info_cache(
+            topicTalosResourceName=self.topicTalosResourceName.topicTalosResourceName,
+            talosClientConfig=producerConfig,
+            messageClient=self.talosClientFactory.new_message_client(),
+            talosClientFactory=self.talosClientFactory)
+        self.falconWriter = FalconWriter(
+            falconUrl=self.talosProducerConfig.get_falcon_url())
 
         # getTopicTalosResourceName(producerConfig, credential);
         self.check_and_get_topic_info(self.topicTalosResourceName.topicTalosResourceName)
 
         self.init_partition_sender()
         self.init_check_partition_task()
+        self.init_producer_monitor_task()
 
         self.logger.info("Init a producer for topic: " +
                          self.topicTalosResourceName.topicTalosResourceName +
@@ -97,10 +111,7 @@ class TalosProducer(object):
             response = self.talosAdmin.get_topic_attribute(
                 GetTopicAttributeRequest(topicName=self.topicName))
         except Exception as e:
-            if isinstance(e, GalaxyTalosException):
-                self.logger.error(e.errMsg)
-            self.logger.error("Exception in CheckPartitionTask: "
-                              + str(traceback.format_exc()))
+            self.logger.error("Exception in CheckPartitionTask: " + e.message)
             if Utils.is_topic_not_exist(e):
                 self.disable_producer(e)
             return
@@ -124,12 +135,23 @@ class TalosProducer(object):
             # update partitionNumber
             self.set_partition_number(topicPartitionNum)
 
-        self.partitionCheckExecutor = threading.Timer(interval=self.talosProducerConfig.
-                                                      get_producer_check_partition_interval(),
+        self.partitionCheckExecutor = threading.Timer(interval=self.talosProducerConfig
+                                                      .get_producer_check_partition_interval(),
                                                       function=self.check_partition_task)
-        self.partitionCheckExecutor.setName("partitionCheckTask-" + self.topicName)
+        self.partitionCheckExecutor.setName("talos-producer-partitionCheck-" + self.topicName)
         self.partitionCheckExecutor.start()
 
+    def producer_monitor_task(self):
+        try:
+            self.push_metric_data()
+        except Exception as throwable:
+            self.logger.error("push metric data to falcon failed." + str(traceback.format_exc()))
+        self.producerMonitorThread = threading.Timer(interval=self.talosProducerConfig
+                                                     .get_report_metric_interval_millis() / 1000,
+                                                     function=self.producer_monitor_task)
+        self.producerMonitorThread.setName("talos-producer-monitor-" + self.topicName)
+        self.producerMonitorThread.start()
+
     @synchronized
     def add_user_message(self, msgList=None, partitionId=None):
         if not self.is_active():
@@ -150,10 +172,8 @@ class TalosProducer(object):
                     self.globalLock.wait()
                     self.globalLock.release()
                 except Exception as e:
-                    if isinstance(e, GalaxyTalosException):
-                        self.logger.error(e.errMsg)
                     self.logger.error("addUserMessage global lock wait is interrupt."
-                                      + str(traceback.format_exc()))
+                                      + e.message)
 
         if not partitionId:
             self.do_add_user_message(msgList)
@@ -256,6 +276,8 @@ class TalosProducer(object):
         for key, value in self.partitionSenderMap.items():
             value.shutdown()
         self.partitionCheckExecutor.cancel()
+        self.producerMonitorThread.cancel()
+        self.scheduleInfoCache.shut_down(self.topicTalosResourceName)
 
     def adjust_partition_sender(self, newPartitionNum=None):
         if self.partitionNumber < newPartitionNum:
@@ -304,7 +326,7 @@ class TalosProducer(object):
                                        + " not found")
 
         self.partitionNumber = response.topicAttribute.partitionNumber
-        self.curPartitionId = random.randint(0, self.partitionNumber)
+        self.curPartitionId = random.randint(0, self.partitionNumber-1)
         self.topicTalosResourceName = topicTalosResourceName
 
     def init_partition_sender(self):
@@ -318,13 +340,27 @@ class TalosProducer(object):
         self.partitionCheckExecutor = Timer(interval=self.talosProducerConfig.
                                             get_producer_check_partition_interval(),
                                             function=self.check_partition_task)
-        self.partitionCheckExecutor.setName("partitionCheckTask-" + self.topicName)
+        self.partitionCheckExecutor.setName("talos-producer-partitionCheckTask-" + self.topicName)
         self.partitionCheckExecutor.start()
 
+    def init_producer_monitor_task(self):
+        if self.talosProducerConfig.is_open_client_monitor():
+            # push metric data to falcon every minutes
+            self.producerMonitorThread = Timer(interval=self.talosProducerConfig.
+                                               get_report_metric_interval_millis() / 1000,
+                                               function=self.producer_monitor_task)
+            self.producerMonitorThread.setName("talos-producer-monitor-" + self.topicName)
+            self.producerMonitorThread.start()
+
     def increase_buffered_count(self, incrementNumber=None, incrementBytes=None):
         self.bufferedCount.increase(incrementNumber, incrementBytes)
 
     def decrease_buffered_count(self, decrementNumber=None, decrementBytes=None):
         self.bufferedCount.decrease(decrementNumber, decrementBytes)
 
+    def push_metric_data(self):
+        jsonArray = []
+        for key, value in self.partitionSenderMap.items():
+            jsonArray += value.get_falcon_data()
+        self.falconWriter.push_falcon_data(jsonArray)
 
